package org.relaxng.datatype;

/**
 * Datatype object.
 * 
 * Every Datatype object has several functionalities. Particularly,
 * 
 * <ol>
 *  <li> ability to identifies a class of character sequences. This is
 *       done through allows method.
 * 
 *  <li> ability to produce "value object" from a character sequence and
 *		 a context information.
 * 
 *  <li> ability to test equality of two value objects.
 * </ol>
 * 
 * This interface also defines optional diagnose method, which provides
 * human-friendly error diagnosis.
 * 
 * @author <a href="mailto:kohsuke.kawaguchi@eng.sun.com">Kohsuke KAWAGUCHI</a>
 */
public interface Datatype {
	
	/**
	 * checks if 'literal' matchs this Datatype under the current context.
	 * 
	 * @param literal
	 *		the lexical representation to be verified
	 * @param context
	 *		context information that may be
	 *		necessary to verify the given literal.
	 * 
	 * @return
	 *		true if 'literal' is a member of this Datatype;
	 *		false if it's not a member of this Datatype.
	 */
	boolean allows( String literal, ValidationContext context );
	
	/**
	 * diagnoses the reason of error.
	 * 
	 * Application can call this method to provide detailed error message to users.
	 * This method is kept separate from verify method to achieve higher performance
	 * during normal validation. This method is optional and may not be implemented
	 * by some of Datatypes.
	 * 
	 * @exception DatatypeException
	 *		If the callee supports the diagnosis and the given literal is invalid,
	 *		then this exception is thrown, possibly with diagnostic information.
	 */
	void check( String literal, ValidationContext context )
		throws DatatypeException;
	
	/**
	 * creates an instance of the streaming validator for this type.
	 * 
	 * By using streaming validators instead of the accepts method,
	 * the caller can avoid keeping the entire string into the memory,
	 * which is sometimes quite big.
	 * 
	 * <p>
	 * This method is optional and implementations are free to throw
	 * UnsupportedOperationException to indicate that this method is
	 * not implemented.
	 * 
	 * @param context
	 *		context information that may be
	 *		necessary to verify the given literal.
	 *		The callee may keep a reference to this context object
	 *		while the returned streaming validator is being used.
	 * 
	 * @exception UnsupportedOperationException
	 *		if streaming validation is not supported by the callee.
	 */
	DatatypeStreamingValidator createStreamingValidator( ValidationContext context )
		throws UnsupportedOperationException;
	
	/**
	 * converts lexcial value and the current context to the corresponding
	 * value object.
	 * 
	 * <p>
	 * The caller cannot generally assume that the value object is a meaningful
	 * Java object. For example, the caller cannot expect this method to return
	 * java.lang.Number type for the "integer" type of XML Schema Part2.
	 * 
	 * <p>
	 * The returned value object should be used solely for the sameValue method.
	 * 
	 * @return	null
	 *		when the given lexical value is not a valid lexical value for this type.
	 */
	Object createValue( String literal, ValidationContext context );
	
	/**
	 * tests the equality of two value objects which belongs to this type.
	 * 
	 * The bahavior is undefined if objects not belonged to this type is passed.
	 * It is the caller's responsibility to ensure that objects
	 * belong to this type.
	 * 
	 * @return
	 *		true if two value objects are considered equal;
	 *		false if otherwise.
	 */
	boolean sameValue( Object value1, Object value2 );
	
	
	/**
	 * computes the hash code for a value object,
	 * which is consistent with the sameValue method.
	 * 
	 * <p>
	 * The application should note that the hashCode method of the value object
	 * is not necessarily consistent with the sameValue method.
	 * Care has to be taken when you are planning to store the value objects
	 * into <code>java.util.Hashtable</code> or something.
	 * 
	 * @return
	 *		hash code for the specified value object.
	 */
	int valueHashCode( Object value );
}
